@@ -15,7 +15,10 @@ class MeinheldWorker(Worker):
         fd = self.socket.fileno()
         server.set_keepalive(self.cfg.keepalive)
         server.set_picoev_max_fd(self.cfg.worker_connections)
-        server.set_watchdog(self.watchdog)
+        
+        server.set_fastwatchdog(self.tmp.fileno(), self.ppid)
+        #server.set_watchdog(self.watchdog)
+        
         server.set_listen_socket(fd)
         server.run(self.wsgi)
 
