@@ -15,6 +15,12 @@
 #define LIMIT_REQUEST_FIELD_SIZE 1024 * 8 
 
 
+typedef enum {
+    BODY_TYPE_NONE,
+    BODY_TYPE_TMPFILE,
+    BODY_TYPE_BUFFER
+} request_body_type;
+
 typedef enum {
     FIELD,
     VALUE,
@@ -33,8 +39,36 @@ typedef struct {
     header *headers[LIMIT_REQUEST_FIELDS];
     uint32_t num_headers;
     field_type last_header_element;   
+    
+    PyObject *env;
+    void *next;
+    int body_length;
+    int body_readed;
+    int bad_request_code;
+    void *body;
+    request_body_type body_type;
+
 } request;
 
+typedef struct {
+    int size;
+    request *head;
+    request *tail;
+} request_queue;
+
+
+inline void 
+push_request(request_queue *q, request *req);
+
+inline request*
+shift_request(request_queue *q);
+
+inline request_queue*
+new_request_queue(void);
+
+inline void
+free_request_queue(request_queue *q);
+
 inline request *
 new_request(void);
 
