@@ -6,13 +6,12 @@
 #include <sys/un.h>
 #include <sys/stat.h>
 
-#include "cStringIO.h"
-
 #include "http_request_parser.h"
 #include "response.h"
 #include "log.h"
 #include "client.h"
 #include "util.h"
+#include "stringio.h"
 
 #define ACCEPT_TIMEOUT_SECS 1
 #define READ_TIMEOUT_SECS 30 
@@ -185,12 +184,11 @@ clean_cli(client_t *client)
     }
     if(client->body){
         if(client->body_type == BODY_TYPE_TMPFILE){
-            if(client->body){
-                fclose(client->body);
-            }
+            fclose(client->body);
         }else{
-            Py_CLEAR(client->body);
+            free_buffer(client->body);
         }
+        client->body = NULL;
     }
     client->header_done = 0;
     client->response_closed = 0;
@@ -618,7 +616,7 @@ call_wsgi_app(client_t *client, picoev_loop* loop)
 static inline void
 prepare_call_wsgi(client_t *client)
 {
-    PyObject *input = NULL, *object = NULL, *c = NULL;
+    PyObject *input = NULL, *c = NULL;
     char *val;
     
     set_current_request(client);
@@ -659,22 +657,17 @@ prepare_call_wsgi(client_t *client)
         client->body = NULL;
     }else{
         if(client->body_type == BODY_TYPE_BUFFER){
-            object = PycStringIO->cgetvalue((PyObject *)client->body);
-            Py_XDECREF((PyObject *)client->body);
-            input = PycStringIO->NewInput(object);
+            input = StringIOObject_New((buffer *)client->body);
             PyDict_SetItem((PyObject *)client->environ, wsgi_input_key, input);
         }else{
             if(client->body){
-                object = client->body;
+                input = StringIOObject_New((buffer *)client->body);
             }else{
-                object = empty_string;
-                Py_INCREF(empty_string);
+                input = StringIOObject_New(new_buffer(0, 0));
             }
-            input = PycStringIO->NewInput(object);
             PyDict_SetItem((PyObject *)client->environ, wsgi_input_key, input);
         }
-        client->body = object;
-        //Py_DECREF(object);
+        client->body = NULL;;
         Py_DECREF(input);
     }
 
@@ -713,7 +706,7 @@ static void
 r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
 {
     client_t *cli = ( client_t *)(cb_arg);
-    PyObject *body = NULL;
+    //PyObject *body = NULL;
     char *key = NULL;
     int finish = 0, nread;
 
@@ -827,16 +820,21 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
                 if(parser_finish(cli) > 0){
                     if(cli->upgrade){
                         //WebSocket Key
+#ifdef DEBUG
+                        printf("upgrade websocket %d \n", cli->fd);
+#endif
                         key = buf + nread + 1;
-                        body = PyString_FromStringAndSize(key, r - nread);
-                        cli->request_queue->tail->body = body;
-
+                        buffer *b = new_buffer(r - nread -1, r - nread -1);
+                        if(write2buf(b, key, r - nread -1) == WRITE_OK){
+                            cli->request_queue->tail->body = b;
+                        }else{
+                            free_buffer(b);
+                        }
                     }
                     finish = 1;
                 }
                 break;
         }
-
     }
     if(finish == 1){
         picoev_del(loop, cli->fd);
@@ -904,8 +902,8 @@ setup_server_env(void)
     request_list_fill();
     header_list_fill();
     buffer_list_fill();
+    StringIOObject_list_fill();
 
-    PycString_IMPORT;
     PyGreenlet_Import();
     
     hub_switch_value = Py_BuildValue("(i)", -1);
@@ -926,6 +924,7 @@ clear_server_env(void)
     request_list_clear();
     header_list_clear();
     buffer_list_clear();
+    StringIOObject_list_clear();
 
     Py_DECREF(hub_switch_value);
     Py_DECREF(client_key);
@@ -1613,7 +1612,7 @@ meinheld_get_ident(PyObject *self, PyObject *args)
             printf("get thread ident %p\n", pyclient->greenlet);
 #endif
             Py_INCREF(pyclient->greenlet);
-            return pyclient->greenlet;
+            return (PyObject *)pyclient->greenlet;
         }
     }
     Py_RETURN_NONE;
@@ -1675,6 +1674,10 @@ initserver(void)
         return;
     }
 
+    if(PyType_Ready(&StringIOObjectType) < 0){
+        return;
+    }
+
     timeout_error = PyErr_NewException("meinheld.server.timeout",
 					  PyExc_IOError, NULL);
 	if (timeout_error == NULL)
